# pio_iq_speed_test.py - 03.10.2021
#
# Tester, wie lange ein Interrut benötigt, der eine globale 'Vairable
# incrementiert.
# m Takt eines Rechtecksignals wird ein Interrupt 10_000 mal aufgerufen,
# wenn kein Interrupt verlosren geht, dann sollte das Ergebnis 10_000 sein
#
# Das funtioniert bei einem Pin-Interrupt bis 15_000 Hz,
# bei einem Ineterrupt, der aus dem pio_asm-Code aufgerufen wird,
# bis 18_000 Hz
# Das Abarbeiten des vollständigen Interrupts dauert alsp:
# 1 / 15_000 Sek. ca. 66us bzw
# 1 / 18_000 Sek. ca. 55us 

import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin

@asm_pio(set_init=PIO.OUT_LOW, in_shiftdir=PIO.SHIFT_RIGHT)
def pulse_train_sym():
    
    wrap_target()
    
    pull(block)              #   1 warten auf Input(Anzahl der Pulse)
    mov(x, osr)              #   2 Anzahl nach x zum Downcounten
    
    label("next")            #     pulse + pause = 10 Takte 
    set(pins, 1)         [3] #   3 5 Takte
    irq(rel(0))              #     ein Takt von den 5 für den IRQ abgeknapst
    set(pins, 0)         [3] #   4 1 + 3 + 1 = 5 Takte
    jmp(x_dec, "next")       #   5 bedingter jmp für begrenzte Wave
    #jmp("next")             #   5 alternativ für endlose Wave
    
    wrap()

class Wave_Sym(object):
    def __init__(self, statemachine, output_pin, wave_freq, irq_handler = None):
       self.sm = StateMachine(statemachine,
                        pulse_train_sym,
                        freq=wave_freq*10,
                        set_base=Pin(output_pin))
       self.sm.restart()
       if irq_handler is not None:
           self.sm.irq(irq_handler)
       
    def transmit(self, cnt):
        self.sm.active(1)
        self.sm.put(cnt - 1)

    def activate(self, state=1):
        self.sm.active(1 if state else 0)

if __name__ == "__main__":    
    output_pin = 0             # Hier wird ds Signal ausgegeben
    p = Pin(1, Pin.IN)         # hier wird der Pin-Interrupt ausgelöst
    
    wave_freq  = 16000         # mit diesem Wert kann man spielen
    wave_cnt   = 10000         # ANzahl der Interrupt 
    
    # Hier wird innerhalb eines Interrupst ein globaler Counter incrementiert
    # Der Irq-Handler wird entweder von einem Pin.irq oder
    # einem irq(0) innerhalb des pio_asm-Codes aufgerufen.
    counter = 0
    
    # @micropython.native
    def irq_handler(_):
        global counter
        counter += 1
    
    # IRQ-Handler per Pin Interrupt 
    p.irq(irq_handler, trigger = (Pin.IRQ_FALLING) )
    wav = Wave_Sym(0, output_pin=output_pin, wave_freq=wave_freq, irq_handler = None)   
    
    # IRQ-Handler per irq(0) im pio_asm-Code
    #wav = Wave_Sym(0, output_pin=output_pin, wave_freq=wave_freq, irq_handler = irq_handler)
    
    wav.activate()
    wav.transmit(wave_cnt)

    time.sleep(1)   # warten bis die Arbeit abgeschlossen ist
    print("Send: {}, received: {}, lost: {}".format(wave_cnt, counter, wave_cnt - counter))
    print("Pulsfreq: {} Hz, Pulslen: {}".format(wave_freq,  1 / wave_freq))
    print("\ndone")
    wav.activate(0)
    

        
