# -- spi_slave_test_1.py -- 17.10.2021

import time
import array as arr
from spi_slave  import SPI_Slave
from spi_master import SPI_Master

# wieviele Words sollen pro Transaktion übertragen werden [1-8] ?
SPI_WORDS = const(3)  
SPI_BYTES = const(SPI_WORDS * 4)

word_buffer = arr.array("I", [0] * SPI_WORDS)
byte_buffer = arr.array("B", [0] * SPI_WORDS * 4)

master = SPI_Master(mosi_pin=19, miso_pin=16, sck_pin=18, csel_pin=17, spi_words=SPI_WORDS, F_SPI=1_000_000)
slave  = SPI_Slave(csel=28, mosi=26, sck=27, miso=22, spi_words=SPI_WORDS, F_PIO=10_000_000)

# --- Hilfsfunktionen ------------------------------------------

def word_2_bytes(word):
    data = arr.array("B", [0] * 4)
    for i in range(3, -1, -1):
        data[i] = word & 0xFF
        word >>= 8
    return data

def bytes_2_word(data):
    # erwartet ein Tuple, Array, bytearray mit 4 Elementen
    if not len(data) == 4:
        raise ValueError("Expecting 4 Bytes in {}, got {} !".format(data, len(data)))
    word = 0
    for i in data:
        word <<= 8
        word |= i
    return word

# Userfunktion, die die empfangenen Daten ggf. interpretiert und
# neue Daten in die TX-Fifo-Register schreibt.
def user_func():
    # Referenzen auf die Buffer des Slaves für read und write
    read  = slave.rx_words()
    write = slave.tx_words()
    
    # addieren von 1 zu jedem Byte
    for i in range(SPI_WORDS):
        write[i] = read[i] + 0x01010101
        
    # die geänderten Daten stehen bereits in self.tx_buffer
    # und weren in die TX-Fifos übertragen
    slave.put_words()
	
    # die geänderten Daten an den Master schicken,
    # damit er sie in der nächsten Transaktion sendet
    master.write(write)


# --- Tests -----------------------------------

# --- Vorbereitung

record = 0          # die Nummerierung beginnt mit 
mainloop = 5    # nach wievielen Schleifendurchläufe beenden

# der Master schreibt die Daten 1 .. x an den Slave,
# der liest sie ein, addiert zu jedem Byyte eine "1"
# und schreibt die Daten in den Sendepuffer.
# Der MAster empfängt die Daten und sendet sie wieder an den SLave.
# Nach n Runden sollte die Zähler im n incrementiert sein.
for i in range(SPI_BYTES):   # schreibt die Bytes (1, 2, 3, 4, 5 ..)
    byte_buffer[i] = i
master.write(byte_buffer)

# --- mainloop -------------------------------

while mainloop:
    
    #time.sleep(.001)     # etwas Wartezeit einräumen
                          # der Master muss die Daten aufbereiten
                          # und senden
                          # und der Slave muss die Daten aus den
                          # Fifos abholen
    if slave.received():  
        
        print("{}.)".format(record))
        record += 1
        
        print("slave  rx:", end = " ")
        for i in slave.rx_words():
            print("{:08X}".format(i), end = " ")            
         
        print("\nmaster rx:", end = " ")
        for i in master.rx_words():
            print ("{:08X}".format(i), end = " ")
        print()
         
        user_func()  # Daten auswerten und
                     # den TX-Puffer füllen
   
    mainloop -= 1
    
print("\ndone")

"""    --- Ausgabe und Interpretation
0.)
slave  rx: 00010203 04050607 08090A0B 
master rx: AAAAAAAA AAAAAAAA AAAAAAAA 
1.)
slave  rx: 01020304 05060708 090A0B0C 
master rx: 01020304 05060708 090A0B0C 
2.)
slave  rx: 02030405 06070809 0A0B0C0D 
master rx: 02030405 06070809 0A0B0C0D 
3.)
slave  rx: 03040506 0708090A 0B0C0D0E 
master rx: 03040506 0708090A 0B0C0D0E 
4.)
slave  rx: 04050607 08090A0B 0C0D0E0F 
master rx: 04050607 08090A0B 0C0D0E0F

0.)
Der Master sendet 0x01020304 .......
und erhält vom Slave 0xAAAA_AAAA als Dummy-Daten zurück (der TX-Buffer war leer).

Die user_func() des Slave addiert zu den empfangenen Daten 0x01010101 und schreibt
die neuen Daten in die TX-Fifo-Register.

1.)
Der Master sendet die vom Slave empfangenen Daten 1:1 zurück an den Slave.
Der addiert wieder 0x01010101 und packt die Daten in die TX-Fifo-Register

2.)
Der Master sendet die vom Slave empfangenen Daten 1:1 zurück an den Slave.
Der addiert wieder 0x01010101 und packt die Daten in die TX-Fifo-Register

...
Ergebnis
Mit jeder Transaktion werden die Bytes um 1 incrementiert.

"""
