# -- spi_slave_test_3.py -- 16.10.2021

import time
import array as arr
from spi_slave  import SPI_Slave
from spi_master import SPI_Master

# wieviele Words sollen pro Transaktion übertragen werden [1-8] ?
SPI_WORDS = const(3)  
SPI_BYTES = const(SPI_WORDS * 4)

word_buffer = arr.array("I", [0] * SPI_WORDS)
byte_buffer = arr.array("B", [0] * SPI_WORDS * 4)

master = SPI_Master(mosi_pin=19, miso_pin=16, sck_pin=18, csel_pin=17, spi_words=SPI_WORDS, F_SPI=1_000_000)
slave  = SPI_Slave(csel=28, mosi=26, sck=27, miso=22, spi_words=SPI_WORDS, F_PIO=10_000_000)

# --- Hilfsfunktionen ------------------------------------------

def word_2_bytes(word):
    data = arr.array("B", [0] * 4)
    for i in range(3, -1, -1):
        data[i] = word & 0xFF
        word >>= 8
    return data

def bytes_2_word(data):
    # erwartet ein Tuple, Array, bytearray mit 4 Elementen
    if not len(data) == 4:
        raise ValueError("Expecting 4 Bytes in {}, got {} !".format(data, len(data)))
    word = 0
    for i in data:
        word <<= 8
        word |= i
    return word

# Userfunktion, die die empfangenen Daten ggf. interpretiert und
# neue Daten in die TX-Fifo-Register schreibt.
messwert = 0x10
def user_func():
    global messwert
    # Referenzen auf die Buffer des Slaves für read und write
    # Der Readbuffer wird ignoriert.
    #read  = slave.rx_words()
    write = slave.tx_words()
    
    # schreibt die "Messwerte" als 4 Bytes jeweils in ein Word
    for i in range(SPI_WORDS):
        d = [0, 0, 0, 0]
        for j in range(4):
            d[j] = messwert & 0xFF
            messwert += 1
        write[i] = bytes_2_word(d)
            
    # die Daten stehen bereits in self.tx_buffer
    # und weren in die TX-Fifos übertragen
    slave.put_words()


# --- Tests -----------------------------------

# --- Vorbereitung

record = 0          # die Nummerierung beginnt mit 
mainloop = 5    # nach wievielen Schleifendurchläufe beenden

# Die TX-Fifo-Register werden hier vorab mit Werten gefüllt
user_func()

# der Master will Daten vom Slave abholen und schreibt
# Dummy-Daten.
# Er wird die gleiche Anzahl an Bytes vom Slave zurückerhalten.
# Ds sind diejenigen Daten, die oben die user_func() in die
# TX-Fifos des Salve geschrieben hat.
master.read()

# --- mainloop -------------------------------

while mainloop:
    if slave.received():  
        
        print("{}.)".format(record))
        record += 1
        
        print("slave  rx:", end = " ")
        for i in slave.rx_words():
            print("{:08X}".format(i), end = " ")            
         
        print("\nmaster rx:", end = " ")
        for i in master.rx_words():
            print ("{:08X}".format(i), end = " ")
        print()
         
        user_func()  # Daten auswerten und
                     # den TX-Puffer füllen
                     
        # Für die Demo den SPI-Master die nächste
        # Transaktion abschicken lassen
        master.read()
   
    mainloop -= 1
    
print("\ndone")

"""    --- Ausgabe und Interpretation
0.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: 10111213 14151617 18191A1B 
1.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: 1C1D1E1F 20212223 24252627 
2.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: 28292A2B 2C2D2E2F 30313233 
3.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: 34353637 38393A3B 3C3D3E3F 
4.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: 40414243 44454647 48494A4B 

0.)
Der Master sendet 0xAAAA_AAAA als Dummy-Daten
und erhält vom Slave die ersten Messwerte (die vorher in TX-Buffer kopiert wurden.

Die user_func() des Slave schreibt nun die nächsten "Messwerte" als Bytes in die
Words und kopiert die neuen Daten in die TX-Fifo-Register.

1.)
Der Master sendet 0xAAAA_AAAA als Dummy-Daten
und erhält vom Slave die nächsten Messwerte als Bytes in Words verpackt.

Die user_func() des Slave schreibt nun die nächsten "Messwerte" als Bytes in die
Words und kopiert die neuen Daten in die TX-Fifo-Register.

...
Ergebnis
Mit jeder Transaktion erhält der Master neue Daten (4 Bytes pro Word) ausgeliefert.

"""
