# -- spi_slave_test_4.py -- 17.10.2021

import time
import array as arr
from spi_slave  import SPI_Slave
from spi_master import SPI_Master

# wieviele Words sollen pro Transaktion übertragen werden [1-8] ?
SPI_WORDS = const(3)  
SPI_BYTES = const(SPI_WORDS * 4)

word_buffer = arr.array("I", [0] * SPI_WORDS)
byte_buffer = arr.array("B", [0] * SPI_WORDS * 4)

master = SPI_Master(mosi_pin=19, miso_pin=16, sck_pin=18, csel_pin=17, spi_words=SPI_WORDS, F_SPI=1_000_000)
slave  = SPI_Slave(csel=28, mosi=26, sck=27, miso=22, spi_words=SPI_WORDS, F_PIO=10_000_000)

# --- Hilfsfunktionen ------------------------------------------

def word_2_bytes(word):
    data = arr.array("B", [0] * 4)
    for i in range(3, -1, -1):
        data[i] = word & 0xFF
        word >>= 8
    return data

def bytes_2_word(data):
    # erwartet ein Tuple, Array, bytearray mit 4 Elementen
    if not len(data) == 4:
        raise ValueError("Expecting 4 Bytes in {}, got {} !".format(data, len(data)))
    word = 0
    for i in data:
        word <<= 8
        word |= i
    return word

# Userfunktion, die die empfangenen Daten ggf. interpretiert und
# neue Daten in die TX-Fifo-Register schreibt.
messwert = 0x10
def user_func():
    global messwert

    # Hier wird eine neue Liste angelegt und später
    # an slave.put_words() übergeben
    messwerte = [0] * SPI_WORDS
    
    # schreibt die "Messwerte" als Bytes in die Words
    for i in range(SPI_WORDS):
        d = [0, 0, 0, 0]
        for j in range(4):
            d[j] = messwert & 0xFF
            messwert += 1
        messwerte[i] = bytes_2_word(d)
            
    # die Daten stehen in messerte[] und werden in
	# die TX-Fifo-Register kopoert
	
    slave.put_words(messwerte)


# --- Tests -----------------------------------

# --- Vorbereitung

record = 0          # die Nummerierung beginnt mit 
mainloop = 5    # nach wievielen Schleifendurchläufe beenden

# Die TX-Fifo-Register werden vorab gefüllt
user_func()

# der Master will Daten vom Slave abholen und schreibt
# Dummy-Daten.
# Er wrid die gleiche Anzahl an Bytes vom Slave zurückerhalten.
master.read()

# --- mainloop -------------------------------

while mainloop:
    
    #time.sleep(.001)     # etwas Wartezeit einräumen
                          # der Master muss die Daten aufbereiten
                          # und senden
                          # und der Slave muss die Daten aus den
                          # Fifos abholen
    if slave.received():  
        
        print("{}.)".format(record))
        record += 1
        
        print("slave  rx:", end = " ")
        for i in slave.rx_words():
            print("{:08X}".format(i), end = " ")            
         
        print("\nmaster rx:", end = " ")
        for i in master.rx_words():
            print ("{:08X}".format(i), end = " ")
        print()
         
        user_func()  # Daten auswerten und
                     # den TX-Puffer füllen
                     
        # Für die Demo den SPI-Master die nächste
        # Transaktion abschicken lassen
        master.read()
   
    mainloop -= 1
    
print("\ndone")

"""    --- Ausgabe und Interpretation

wie spi_slave_test_3.py

"""
