# -- spi_slave_test_5.py -- 17.10.2021

import time
import array as arr
from spi_slave  import SPI_Slave
from spi_master import SPI_Master

# wieviele Words sollen pro Transaktion übertragen werden [1-8] ?
SPI_WORDS = const(8)  
SPI_BYTES = const(SPI_WORDS * 4)

word_buffer = arr.array("I", [0] * SPI_WORDS)
byte_buffer = arr.array("B", [0] * SPI_BYTES)

master = SPI_Master(mosi_pin=19, miso_pin=16, sck_pin=18, csel_pin=17, spi_words=SPI_WORDS, F_SPI=1_000_000)
slave  = SPI_Slave(csel=28, mosi=26, sck=27, miso=22, spi_words=SPI_WORDS, F_PIO=10_000_000)

# --- Hilfsfunktionen ------------------------------------------

# Userfunktion, die die empfangenen Daten ggf. interpretiert und
# neue Daten in die TX-Fifo-Register schreibt.
# @micropython.native
def user_func():
    # Referenzen auf die Buffer für read und write
    read  = slave.rx_words()
    write = slave.tx_words()
    
    # invertiert die Bytes
    for i in range(SPI_WORDS):
        write[i] = ~read[i]

# --- Tests -----------------------------------

# --- Vorbereitung

record = 0          # die Nummerierung beginnt mit 

# der Master schreibt Daten die Daten 0xAAAA_AAAA an den Slave,
# der liest sie ein und schreibt die invertierten Werte in den TX-Buffer.
# Bitfehler bei der Übertragung kann man am Bitmuster gut erkennen

for i in range(SPI_WORDS):
    word_buffer[i] = 0xAAAA_AAAA
    
for i in range(SPI_BYTES):  
    byte_buffer[i] = 0xAA

reads = 6        #  so viele mal schreiben

# für die Ausgabe der mittleren Laufzeit die Anzahl merken
_reads = reads

starttime = time.ticks_us()
master.read()
# master.write(word_buffer)

# --- mainloop -------------------------------

while True:   
    
    #time.sleep(.001)     # etwas Wartezeit einräumen
                          # der Master muss die Daten aufbereiten
                          # und senden
                          # und der Slave muss die Daten aus den
                          # Fifos abholen
    
    if slave.received():  
        
        # """# Speedtest ohne Ausgabe !

        print("{}.)".format(record))
        record += 1
        
        print("slave  rx:", end = " ")
        for i in slave.rx_words():
            print("{:08X}".format(i), end = " ")            
         
        print("\nmaster rx:", end = " ")
        for i in master.rx_words():
            print ("{:08X}".format(i), end = " ")
        print()
        
        # """
        
        user_func()  # ggf. die empfangenen Daten auswerten,
                     # den TX-Puffer füllen 
                     
        # den TX_Buffer in die TX-Fifo-Register kopieren
        # Beim nächsten Read werden sie vom Master abgeholt
        slave.put_words()
        
        # Daten an dem Master schicken,
        # damit er sie in der nächsten Transaktion zurück sendet
        
        master.write(slave.tx_words())     # Übergabe eines Arrays auf dem Slave ~2600 us
        #master.write(word_buffer)          # Übergabe von 8 Words -2000 us
        #master.write(byte_buffer)          # Übergabe von 32 Byte -1600 us
        #master.read()                       # Der Master generiert Dummy-Daten  ~ 1400 is
        
        reads -= 1
        if reads == 0:
            break

read_time = time.ticks_diff(time.ticks_us(), starttime)
print("\nTest über {} Reads".format(_reads))
print("Dauer pro Read: {} us".format(read_time // _reads ))

print("\ndone")

"""    --- Ausgabe und Interpretation
0.)
slave  rx: 55555555 55555555 55555555 55555555 55555555 55555555 55555555 55555555 
master rx: AAA22AAA AEEAAAA8 8AAAABBA AAAA22AA AAEEAAAA 88AAAABB AAAAA22A AAAEEAAA 
1.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: BAAAAA22 AAAAEEAA AA88AAAA BBAAAAA2 2AAAAEEA AAA88AAA ABBAAAAA 22AAAAEE 
2.)
slave  rx: 55555555 55555555 55555555 55555555 55555555 55555555 55555555 55555555 
master rx: 555DD555 51155557 75555445 5555DD55 55115555 77555544 55555DD5 55511555 
3.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: AAAA88AA AABBAAAA A22AAAAE EAAAA88A AAABBAAA AA22AAAA EEAAAA88 AAAABBAA 
4.)
slave  rx: 55555555 55555555 55555555 55555555 55555555 55555555 55555555 55555555 
master rx: 54455555 DD555511 55557755 55445555 5DD55551 15555775 55544555 55DD5555 

...
Ergebnis
Es treten Bitfehler auf, F_SPI ist zu hoch, obwohl F_PIO = F_SPI * 10.
Bei F_SPI 2_000_000 und F_PIO 10_000_000 sind keine Fehler erkennbar.
-> Die Verbindung zwischen Master und Slave ist nicht optimal.

Bei F_SPI 8_000_000 und F_PIO 120_000_000 sieht das Bild bei meinem Testaufbau so aus:

0.)
slave  rx: 55555555 55555555 55555555 55555555 55555555 55555555 55555555 55555555 
master rx: AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA 
1.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA 
2.)
slave  rx: 55555555 55555555 55555555 55555555 55555555 55555555 55555555 55555555 
master rx: 55555555 55555555 55555555 55555555 55555555 55555555 55555555 55555555 
3.)
slave  rx: AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA 
master rx: AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA AAAAAAAA 
4.)
slave  rx: 55555555 55555555 55555555 55555555 55555555 55555555 55555555 55555555 
master rx: 55555555 55555555 55555555 55555555 55555555 55555555 55555555 55555555 

"""
